/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "adxl345.h"
#include "gpio.h"
#include "adc.h"
#include "s25fl004.h"
#include "ls013b4dn04.h"
#include "i2c.h"
#include "lm75b.h"
#include "pcf8885.h"
#include "pmu.h"
#include "clkconfig.h"
#include "wdt.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define KEEP_SWD_ACTIVE

/******************************************************************************
 * Local variables
 *****************************************************************************/

static char messageString[] = "# of wakeups:xx";

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void prepareIoForSleep(void)
{
#if 0
  volatile uint32_t* pIOCON;

  for (	pIOCON = &LPC_IOCON->PIO0_1;
      pIOCON < &LPC_IOCON->PIO1_31;
      pIOCON++)
  {
    //Set pins: FUNC=0, Mode=0
    *pIOCON = (0<<0) | (0<<3) | (1<<7);
  }
  LPC_IOCON->PIO0_4            = (1<<0) | (0<<3);// | (1<<10);  //I2C-SCL, normal mode
  LPC_IOCON->PIO0_5            = (1<<0) | (0<<3);// | (1<<10);  //I2C-SDA, normal mode
#ifdef KEEP_SWD_ACTIVE
  LPC_IOCON->SWCLK_PIO0_10	 = (0<<0) | (2<<3);  //SWCLK, pullup active
#else
  LPC_IOCON->SWCLK_PIO0_10	 = (1<<0) | (0<<3);  //GPIO (not SWCLK), no pullup/down
#endif
  LPC_IOCON->TDI_PIO0_11	     = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TDI), no pullup/down
  LPC_IOCON->TMS_PIO0_12	     = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TMS), no pullup/down
  LPC_IOCON->TDO_PIO0_13       = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TDO), no pullup/down
  LPC_IOCON->TRST_PIO0_14      = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TRST), no pullup/down
#ifdef KEEP_SWD_ACTIVE
  LPC_IOCON->SWDIO_PIO0_15	 = (0<<0) | (2<<3) | (1<<7);  //SWDIO), pullup active
#else
  LPC_IOCON->SWDIO_PIO0_15	 = (1<<0) | (0<<3) | (1<<7);  //GPIO (not SWDIO), no pullup/down
#endif


#else
  //Set pins
  LPC_IOCON->RESET_PIO0_0		 = (0<<0) | (0<<3);  //RESET input, no pullup/down
  LPC_IOCON->PIO0_1            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_2            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_3            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_4            = (1<<0) | (0<<3);// | (1<<10);  //I2C-SCL, normal mode
  LPC_IOCON->PIO0_5            = (1<<0) | (0<<3);// | (1<<10);  //I2C-SDA, normal mode
  LPC_IOCON->PIO0_6            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_7            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_8            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_9            = (0<<0) | (0<<3);  //GPIO, no pullup/down
#ifdef KEEP_SWD_ACTIVE
  LPC_IOCON->SWCLK_PIO0_10	 = (0<<0) | (2<<3);  //SWCLK, pullup active
#else
  LPC_IOCON->SWCLK_PIO0_10	 = (1<<0) | (0<<3);  //GPIO (not SWCLK), no pullup/down
#endif
  LPC_IOCON->TDI_PIO0_11	     = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TDI), no pullup/down
  LPC_IOCON->TMS_PIO0_12	     = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TMS), no pullup/down
  LPC_IOCON->TDO_PIO0_13       = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TDO), no pullup/down
  LPC_IOCON->TRST_PIO0_14      = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TRST), no pullup/down
#ifdef KEEP_SWD_ACTIVE
  LPC_IOCON->SWDIO_PIO0_15	 = (0<<0) | (2<<3) | (1<<7);  //SWDIO), pullup active
#else
  LPC_IOCON->SWDIO_PIO0_15	 = (1<<0) | (0<<3) | (1<<7);  //GPIO (not SWDIO), no pullup/down
#endif
  LPC_IOCON->PIO0_16           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_17           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_18           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_19           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_20           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_21           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_22           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO0_23           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_0            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_1            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_2            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_3            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_4            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_5            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_6            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_7            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_8            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_9            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_10           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_11           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_12           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_13           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_14           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_15           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_16           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_17           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_18           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_19           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_20           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_21           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_22           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_23           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_24           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_25           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_26           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_27           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_28           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_29           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_30           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  LPC_IOCON->PIO1_31           = (0<<0) | (0<<3);  //GPIO, no pullup/down
#endif
  //P0.11/12/13/14 (irq:s) and P0.16 (wakeup) are inputs
  LPC_GPIO->DIR[0] = ~((1<<11)|(1<<12)|(1<<13)|(1<<14)|(1<<16));
  LPC_GPIO->DIR[1] = 0xFFFFFFFF;

  //All pins output low, except P0.6/(P0.8 LCD ENA)/P1.13/(P1.14 LCD PWR)/P1.15/P1.16/P1.23 that shall be high
  LPC_GPIO->CLR[0] = ~((1<<6)|(1<<8));
  LPC_GPIO->CLR[1] = ~((1<<13)|(1<<15)|(1<<16));
  LPC_GPIO->SET[0] = ((1<<6)|(1<<8));
  LPC_GPIO->SET[1] = (1<<13)|(1<<15)|(1<<16);
  //  LPC_SYSCON->PDSLEEPCFG = (1<<6); //BOD powered, WDT powered down
  LPC_SYSCON->PDSLEEPCFG = (1<<3)|(1<<6); //BOD and WDT powered down
}

static void writeCounterOnLCD(void)
{
  uint32_t value;

  value = LPC_PMU->GPREG3;
  messageString[13] = '0' + (value / 10);
  messageString[14] = '0' + (value % 10);
  value = ((value + 1) % 100);
  LPC_PMU->GPREG3 = value;

  lcd_update_vcom(); //Update VCOM
  if (value & 1)
    lcd_update_vcom(); //Update VCOM
  lcd_putString( 0, 35, messageString, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
}

static void PMU_EnterDeepPowerDown( void )
{
  LPC_SYSCON->PDRUNCFG &= ~(IRC_OUT_PD | IRC_PD);  //Power-up IRC

  SCB->SCR |= NVIC_LP_SLEEPDEEP;
  LPC_PMU->PCON = 0x3;
  __WFI();
  return;
}

/******************************************************************************
 * Main function
 *****************************************************************************/

int main(void)
{
  GPIOInit();

  //Set LED pin (PIO0_7) to output and turn LED off
  EA_LED_INIT;
  EA_LED_OFF;

  //Turn on-chip USB peripheral block off
  LPC_SYSCON->PDRUNCFG |= (1 << 10);	//Power-down USB PHY
  LPC_SYSCON->PDRUNCFG |= (1 <<  8);	//Power-down USB PLL

  PMU_Init();

  //Turn LCD off
  PWR_LCD_INIT;
  PWR_LCD_ON;

  //Turn SPI-FLASH off
  PWR_FLASH_INIT;
  PWR_FLASH_OFF;

  //Init and enable I2C pull-up resistors
  PWR_I2C_INIT;
  PWR_I2C_ON;

  //Init pin that controls voltage to trimming potentiometer
  PWR_ADC_INIT;
  PWR_ADC_OFF;

  //check if wakeup from reset or deep power down
  if ((LPC_PMU->PCON & (0x1<<11)) == 0)
  {
    //It was a reset

    if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
    {
      while ( 1 );        /* Fatal error */
    }

    //put capacitive sensor to sleep
    pcf8885_init();
    PCF8885_SLEEP_PIN_HIGH;

    //Init accelerometer
    adxl345_init();
    adxl345_setPowerControl(0x00);

    //Init the temperature sensor
    lm75b_config(LM75B_CFG_SHUTDOWN);

    lcd_init();
    lcd_putString( 0,  4, "Deep power down", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
    lcd_putString( 0, 12, "demo. Press", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
    lcd_putString( 0, 20, "wakeup-key...", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

    //Store counter value
    LPC_PMU->GPREG0 = 0xdeadbeef;
    LPC_PMU->GPREG1 = 0x87654321;
    LPC_PMU->GPREG2 = 0xcafebabe;
    LPC_PMU->GPREG3 = 0;
  }
  else
  {
    //put capacitive sensor to sleep
    PCF8885_SLEEP_PIN_HIGH;

    //Need to initialize the frame buffer data structure
    lcd_init();

    //Actually need to init frame buffer in mcu ram
    //If only part of the LCD was updated (where the counter is written)
    //then the whole display memory would not need an update...
    if(1)
    {
      lcd_putString( 0,  4, "Deep power down", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      lcd_putString( 0, 12, "demo. Press", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      lcd_putString( 0, 20, "wakeup-key...", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
    }

    //it was a deep power down... clear the PCON bit
    LPC_PMU->PCON |= (0x1<<11);

    //check if stored counter value is valid
    if ((LPC_PMU->GPREG0 != 0xdeadbeef)||(LPC_PMU->GPREG1 != 0x87654321)||(LPC_PMU->GPREG2 != 0xcafebabe))
    {
      lcd_putString( 0,  80, "Fail in regs!!!", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
      LPC_PMU->GPREG0 = 0xdeadbeef;
      LPC_PMU->GPREG1 = 0x87654321;
      LPC_PMU->GPREG2 = 0xcafebabe;
      LPC_PMU->GPREG3 = 0;
    }
    else
      lcd_putString( 0,  80, "               ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
  }

  writeCounterOnLCD();

  prepareIoForSleep();

  PMU_EnterDeepPowerDown();

  return 0;
}


