/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "general.h"
#include "application.h"
#include "ls013b4dn04.h"
#include "adxl345.h"


/******************************************************************************
 * Defines, macros, and typedefs
 *****************************************************************************/
// Screen width
#define WIDTH 96
// Screen height
#define HEIGHT 96
// Radius of the player's ball
#define RADIUS 5
// Slowdown due to friction, etc.
#define SLOWDOWN 910
// Slowdown due to collision
#define COLLISIONSLOWDOWN 996


/******************************************************************************
 * Public variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/
static int16_t midScale[3];
static int16_t buffer[3];

// Player's position * 1000
static int32_t gXPos;
static int32_t gYPos;

// Player's motion vector * 1000
static int32_t gXMov;
static int32_t gYMov;

static void
init()
{
  int32_t resBuffer[3];
  uint8_t i;

  gXPos = (WIDTH  / 2) * 1000;
  gYPos = (HEIGHT / 2) * 1000;

  gXMov = 0;
  gYMov = 0;

  //get midscale of accelerometer (only consider x&y values)
  resBuffer[0] = 0;
  resBuffer[1] = 0;
  //get 32 samples
  for(i=0; i<32; i++)
  {
	  adxl345_getOutput((uint16_t*)&buffer[0]);
      resBuffer[0] += buffer[0];
      resBuffer[1] += buffer[1];
  }
  midScale[0] = resBuffer[0] / 32;
  midScale[1] = resBuffer[1] / 32;
}

/******************************************************************************
 * Public functions
 *****************************************************************************/

void applicationBall(void)
{
	static int16_t filtX=0, filtY=0;

	//powerup accelerometer
	adxl345_setDataFormatControl(0x2B);		//Full resolution, +/-16g, 4mg/LSB.
	adxl345_setDataRate(ADXL345_3200HZ);		//3.2kHz data rate.
	adxl345_setPowerControl(MeasurementMode);	//Measurement mode.

	init();
	do
	{
		adxl345_getOutput((uint16_t*)&buffer[0]);
		buffer[0] -= midScale[0];
		buffer[1] -= midScale[1];

#if 1
		filtX = (filtX*7 + buffer[0])>>3;
		filtY = (filtY*7 + buffer[1])>>3;
		buffer[0] = filtX;
		buffer[1] = filtY;
#endif
		if ((buffer[0] < 8) && (buffer[0] > -8))
			buffer[0] = 0;
		if ((buffer[1] < 8) && (buffer[1] > -8))
			buffer[1] = 0;

		gXMov += buffer[0]*15;
		gYMov -= buffer[1]*15;

		gXMov = (gXMov * SLOWDOWN) / 1000;
		gYMov = (gYMov * SLOWDOWN) / 1000;

		gXPos += gXMov;
		gYPos += gYMov;

		// Collision with the screen borders
		if (((gXPos+500)/1000) > WIDTH - RADIUS - 3)
		{
			gXPos -= gXMov;
			gXMov = -(gXMov * COLLISIONSLOWDOWN) / 1000;
		}

		if (((gXPos+500)/1000) < RADIUS + 2)
		{
			gXPos -= gXMov;
			gXMov = -(gXMov * COLLISIONSLOWDOWN) / 1000;
		}

		if (((gYPos+500)/1000) > HEIGHT - RADIUS - 3)
		{
			gYPos -= gYMov;
			gYMov = -(gYMov * COLLISIONSLOWDOWN) / 1000;
		}

		if (((gYPos+500)/1000) < RADIUS + 2)
		{
			gYPos -= gYMov;
			gYMov = -(gYMov * COLLISIONSLOWDOWN) / 1000;
		}

		//start rendering here, create background
		lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
		lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

		// draw the player object
		lcd_circle((gXPos+500)/1000, (gYPos+500)/1000, RADIUS, LCD_COLOR_BLACK, TRUE);
	} while (((LPC_GPIO->PIN[0] & (1<<16)) != 0));	// exit when wakeup button is pressed

	//power down accelerometer
	adxl345_setPowerControl(0x00);
}
