/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/


/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "ls013b4dn04.h"
#include "adxl345.h"
#include "s25fl004.h"
#include "pmu.h"
#include "gpio.h"
#include "pcf8885.h"
#include "pcf8523.h"
#include "pwrapi.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define USE_CAPSENSE_TO_SET_TRANSITION

#define IMG_BUF_SZ (1214)

#define TRANS_NONE	(0)
#define TRANS_1		(1)
#define TRANS_2		(2)
#define TRANS_3		(3)
#define TRANS_4		(4)

/******************************************************************************
 * Global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint8_t buf1[IMG_BUF_SZ];
// put second buffer in unused USB RAM
static uint8_t *buf2 = (uint8_t*)(0x20004000);
static uint16_t tickCnt;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void trans1(uint8_t *img1, uint8_t* img2)
{
  int i = 0;

  for(i=0; i<=96; i+=3)
  {
    lcd_copyImage8(0, 0, &img2[(96*96/8)-(i*96/8)], 96, i, TRUE, FALSE);
    lcd_copyImage8(0, i, &img1[(i*96/8)], 96, 96-i, TRUE, TRUE);
  }
}

static void trans2(uint8_t *img1, uint8_t* img2)
{
  int i = 0;

  for(i=0; i<=96; i+=3)
  {
    lcd_copyImage8(0, 0, &img2[(96*96/8)-(i*96/8)], 96, i, TRUE, FALSE);
    lcd_copyImage8(0, i, &img1[0], 96, 96-i, TRUE, TRUE);
  }
}

static void trans3(uint8_t *pImg1, uint8_t* pImg2)
{
  int i = 96;
  do
  {
    uint8_t colCnt;
    uint8_t rowCnt;
    uint8_t pixels;
    uint8_t pixelsA = 0;
    uint8_t pixelsB = 0;
    uint8_t *pImgA, *pImgB;

    pImgA = &pImg1[0];
    pImgB = &pImg2[0];

    for (rowCnt = 0; rowCnt < 96; rowCnt++)
    {
      for (colCnt = 0; colCnt < 96; colCnt++)
      {
        if ((colCnt & 0x07) == 0)
        {
          pixelsA = *pImgA++;
          pixelsB = *pImgB++;
        }
        if (colCnt < i)
          pixels = pixelsA;
        else
          pixels = pixelsB;
        if (pixels & 0x80)
          lcd_putPixel(colCnt, LCD_DISPLAY_HEIGHT-1-rowCnt, LCD_COLOR_BLACK);
        else
          lcd_putPixel(colCnt, LCD_DISPLAY_HEIGHT-1-rowCnt, LCD_COLOR_WHITE);
        pixelsA <<= 1;
        pixelsB <<= 1;
      }
    }
    lcd_updateScreen(0, 95);


  } while (i-- > 0);

}

static uint8_t getPicturePixel(uint8_t *pImg, uint8_t x, uint8_t y)
{
  const uint8_t bitMask[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
  return pImg[(x/8)+(y*96/8)] & bitMask[x&7];
}

static void trans4(uint8_t *pImg1, uint8_t* pImg2)
{
  int i = 96;
  do
  {
    uint8_t colCnt;
    uint8_t rowCnt;
    uint8_t pixel;

    for (colCnt = 0; colCnt < i; colCnt++)
    {
      for (rowCnt = 0; rowCnt < 96; rowCnt++)
      {
        pixel = getPicturePixel(&pImg1[0],96+colCnt-i,rowCnt);

        if (pixel)
          lcd_putPixel(colCnt, LCD_DISPLAY_HEIGHT-1-rowCnt, LCD_COLOR_BLACK);
        else
          lcd_putPixel(colCnt, LCD_DISPLAY_HEIGHT-1-rowCnt, LCD_COLOR_WHITE);
      }
    }
    for (colCnt = i; colCnt < 96; colCnt++)
    {
      for (rowCnt = 0; rowCnt < 96; rowCnt++)
      {
        pixel = getPicturePixel(&pImg2[0],colCnt-i,rowCnt);

        if (pixel)
          lcd_putPixel(colCnt, LCD_DISPLAY_HEIGHT-1-rowCnt, LCD_COLOR_BLACK);
        else
          lcd_putPixel(colCnt, LCD_DISPLAY_HEIGHT-1-rowCnt, LCD_COLOR_WHITE);
      }
    }
    lcd_updateScreen(0, 95);
  } while (i-- > 0);

}

static uint8_t app_slide_1hz_tick(void)
{
	if (tickCnt > 0)
		tickCnt--;

	return FALSE; //let rtc interrupt handle 1Hz lcd update
}


/******************************************************************************
 * Public functions
 *****************************************************************************/


void applicationSlideshow(void)
{
  uint32_t result = 0;
  uint8_t row = 0;
  uint32_t sectorSize = 0;
  uint8_t *pImgPrev = NULL;
  uint8_t *pImgNext = NULL;
  uint16_t offset = 0;
  uint8_t trans = TRANS_NONE;
#ifdef USE_CAPSENSE_TO_SET_TRANSITION
  uint8_t key = 0;
#endif

  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<27);  //make sure usb ram is enabled

  /* Clear the LCD screen */
  lcd_clearScreen(LCD_COLOR_WHITE, TRUE);

  pImgNext = &buf1[0];

  do {
    uint32_t lastValid = 0xFFFF;
    uint32_t lastFound = FALSE;
    uint32_t nextImg = 1;

    result = s25fl004_init();
    if (result == FALSE) {
      lcd_putString(6, 4+row*8, " Init failed", LCD_COLOR_BLACK,
          LCD_COLOR_WHITE, 25, TRUE);row++;
      break;
    }

    sectorSize = flash_getSectorSize();

    //register 1Hz callback
    pcf8523_registerCallback(app_slide_1hz_tick);

    // exit when wakeup button is pressed
    while((LPC_GPIO->PIN[0] & (1<<16))) {

#ifdef USE_CAPSENSE_TO_SET_TRANSITION
      //activate cap sensors (if inactive)
      PCF8885_SLEEP_PIN_LOW;

      key = pcf8885_readSense();
      if (key == CAP_KEY_OK) {
        trans = TRANS_NONE;
      }
      else if (key == CAP_KEY_3) {
        trans = TRANS_4;
      }
      else if (key == CAP_KEY_2) {
        trans = TRANS_3;
      }
      else if (key == CAP_KEY_1) {
        trans = TRANS_2;
      }
      else if (key == CAP_KEY_0) {
        trans = TRANS_1;
      }
#else
      //put capacitive sensor to sleep
      PCF8885_SLEEP_PIN_HIGH;
#endif

      //powerup flash in order to read next picture
      result = s25fl004_init();
      if (result == FALSE) {
    	  lcd_putString(6, 4+row*8, " Init failed", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);row++;
    	  break;
      }
      result = flash_read(pImgNext, nextImg*sectorSize, IMG_BUF_SZ);
      //power upp flash as quickly as possible to save power
      PWR_FLASH_OFF;
      if (result == 0) {
        lcd_putString(6, 4+row*8, " Load img failed", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);row++;
        break;
      }

      result = lcd_getBmpOffset(pImgNext, &offset);
      if (!result && !lastFound) {
        lastValid = nextImg-1;
        lastFound = TRUE;
      }

      if (pImgPrev != NULL) {
        uint8_t* pTmp = pImgPrev;

        // assuming that all images are identical with regard
        // to the offset

        switch(trans) {
        case TRANS_NONE:
          lcd_copyImageFull(&pImgNext[offset], TRUE, TRUE);
          break;
        case TRANS_1:
          trans1(&pImgPrev[offset], &pImgNext[offset]);
          break;
        case TRANS_2:
          trans2(&pImgPrev[offset], &pImgNext[offset]);
          break;
        case TRANS_3:
          trans3(&pImgPrev[offset], &pImgNext[offset]);
          break;
        case TRANS_4:
          trans4(&pImgPrev[offset], &pImgNext[offset]);
          break;
        }

        pImgPrev = pImgNext;
        pImgNext = pTmp;
      }
      else {

//        lcd_copyImage8(0, 0, &pImgNext[offset], 96, 96, TRUE, TRUE);
        lcd_copyImageFull(&pImgNext[offset], TRUE, TRUE);

        pImgPrev = pImgNext;
        pImgNext = &buf2[0];

      }

      nextImg++;
      if (nextImg > lastValid) {
        nextImg = 1;
      }

      //wait 5 seconds until next transition
      tickCnt = 5;
      while(tickCnt > 0)
      {
    	  //enter sleep
    	  prepareIoForSleep(KEEP_SPI0_ACTIVE, MAKE_CLKOUT_INACTIVE);
    	  PMU_Sleep(MCU_POWER_DOWN, LPC_SYSCON->PDSLEEPCFG & ~(WDT_OSC_PD | BOD_PD));
      }
    }

  } while (0);

  //unregister 1Hz callback
  pcf8523_registerCallback(NULL);


}




