/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/


/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "general.h"
#include "application.h"
#include "gpio.h"
#include "app_systick.h"
#include "ls013b4dn04.h"
#include "pcf8523.h"
#include "adc.h"
#include "systick.h"
#include "pmu.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
static void app_systick_hook(void)
{
	volatile uint32_t i;
	volatile float d;

	PWR_SIGNAL_XXX_START;
	//perform dummy work to load processor
	for(i=0; i<15; i++)
		d = (float)i * 3.14;
	PWR_SIGNAL_XXX_END;
}

static char messageString[5];

static uint8_t app_systick_1hz_tick(void)
{
	static uint16_t adcVal;

    //read ADC input and filter
    adcVal = 4*ADCRead(7);
//    adcVal = (7*adcVal + (4*ADCRead(7))) >> 3;

    if (adcVal < 5)
    	adcVal = 5;

    messageString[0] = '0' + (adcVal / 1000);
    messageString[1] = '0' + ((adcVal / 100) % 10);
    messageString[2] = '0' + ((adcVal / 10) % 10);
    messageString[3] = '0' + (adcVal % 10);
    messageString[4] = 0;

    lcd_update_vcom(); //Update VCOM with 1Hz
    lcd_putString( 42, 20, messageString, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

    initSysTickCounter(adcVal);
    sysTickHookRegister(app_systick_hook);

	return TRUE; //do not let rtc interrupt handle 1Hz lcd update
}
/******************************************************************************
 * Public functions
 *****************************************************************************/

void applicationSystick(void)
{
	//Init pin that controls voltage to trimming potentiometer
	PWR_ADC_INIT;
	PWR_ADC_ON;

	/* Initialize ADC  */
	ADCInit( ADC_CLK );

	/* Clear the LCD screen */
	lcd_clearScreen(LCD_COLOR_WHITE, TRUE);
	lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

	/****************************************************/
	/* Draw menu options                                */
	/****************************************************/
	lcd_putString(6,  4, "Systick vs pow", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_line     (5, 12, 89, 12, LCD_COLOR_BLACK, FALSE);
	lcd_line     (5, 14, 89, 14, LCD_COLOR_BLACK, FALSE);
	lcd_putString(6, 20, "Rate: xxxx Hz", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_putString(6, 31, "Measure curr-", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_putString(6, 39, "ent consumpt-", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_putString(6, 47, "ion relative", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_putString(6, 55, "to rate.", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

	lcd_line     (2, 65, 93, 65, LCD_COLOR_BLACK, FALSE);
	lcd_putString(6, 68, "Return to menu", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_putString(6, 76, "by pressing", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
	lcd_putString(6, 84, "'wakeup' key", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

	initSysTickCounter(5);

    //register 1Hz callback
    pcf8523_registerCallback(app_systick_1hz_tick);

    //register systick hook
    sysTickHookRegister(app_systick_hook);

	// exit when wakeup button is pressed
	while((LPC_GPIO->PIN[0] & (1<<16)))
	{
		//enter sleep
		PMU_Sleep(MCU_SLEEP, LPC_SYSCON->PDSLEEPCFG & ~(WDT_OSC_PD | BOD_PD));
	}

	//stop systick counter
    initSysTickCounter(0);

    //turn off power to trimming pot.
	PWR_ADC_OFF;

	//unregister 1Hz callback
    pcf8523_registerCallback(NULL);

    //unregister systick hook
    sysTickHookRegister(NULL);

}
