/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif


#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "gpio.h"
#include "adc.h"
#include "s25fl004.h"
#include "ls013b4dn04.h"
#include "i2c.h"
#include "lm75b.h"
#include "adxl345.h"
#include "pcf8523.h"
#include "pcf8885.h"
#include "startup.h"
#include "menu.h"
#include "app_clock.h"
#include "app_slideshow.h"
#include "app_ball.h"
#include "app_globe.h"
#include "app_systick.h"
#include "pmu.h"
#include "pwrapi.h"


/******************************************************************************
 * Main function
 *****************************************************************************/

int main(void) {

  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD | ADC_PD);
//  LPC_SYSCON->PDRUNCFG |= (SYS_OSC_PD);

  //have the possibility to watch core activity via PIO0_1
  if (0)
  {
	  LPC_IOCON->PIO0_1     |= (1<<0) | (0<<3);  //CLKOUT, no pullup/down
	  LPC_SYSCON->CLKOUTSEL  = 3;     /* Select main clk Clock Output  */
	  LPC_SYSCON->CLKOUTDIV  = 200;
	  LPC_SYSCON->CLKOUTUEN  = 0x00;  /* Toggle Update Register   */
	  LPC_SYSCON->CLKOUTUEN  = 0x01;
	  LPC_SYSCON->CLKOUTUEN  = 0x00;
  }

  GPIOInit();

  prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);
  LPC_IOCON->PIO0_1 = 0x00;

  //Initialize the power consumption indicator pins
  PWR_SIGNALS_INIT;
  PWR_SIGNALS_RESET;
  PWR_SIGNAL_48MHZ_START;

  /* Set LED pin (PIO0_7) to output */
  EA_LED_INIT;  //LED pin
  EA_LED_OFF;

  PWR_ADC_INIT; //ADC power control pin
  PWR_ADC_OFF;

  PWR_I2C_INIT; //I2C pull-up control pin
  PWR_I2C_ON;

  /* Initialize ADC  */
  ADCInit( ADC_CLK );

  I2CInit( (uint32_t)I2CMASTER );

  //Init the temperature sensor
  lm75b_config(LM75B_CFG_NORMAL);

  //Init the accelerometer
  adxl345_init();
  adxl345_setPowerControl(0x00);

  //Init the RTC
  pcf8523_registerCallback(app_clock_1hz_tick);
  pcf8523_init();

  //Init the capacitive sensors
  pcf8885_init();

  //Init the SPI flash
  s25fl004_init();

  //Init the LCD
  lcd_init();

  //Run startup sequence
  lcd_StartupMessage();

  PMU_Init();

  //Start application
  while(1)
  {
    switch (activateMenu())
    {
    case 0:
      applicationClock();
      break;
    case 1:
      applicationSlideshow();
      break;
    case 2:
      rotatingGlobe();
      break;
    case 3:
      applicationBall();
      break;
    case 4:
      //clock settings
      applicationSetClock();
      break;
    case 5:
      //vary systick
      applicationSystick();
      break;
    case 6:
      break;
    default:
      break;
    }
  }

  return 0 ;
}



