/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/


/******************************************************************************
 * Includes
 *****************************************************************************/

#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "ls013b4dn04.h"
#include "pcf8885.h"
#include "gpio.h"
#include "pmu.h"

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint16_t inactivityCounter = 0;
static uint8_t cursorPos = 0;
static uint8_t keyPressed = FALSE;
static uint8_t keyPressCounter;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void renderMenu(uint8_t oldCursorPos, uint8_t newCursorPos)
{
  uint8_t repaintAll = FALSE;

  if (newCursorPos == oldCursorPos)
    repaintAll = TRUE;

  /* Clear the LCD screen */
  if (TRUE == repaintAll)
  {
    lcd_clearScreen(LCD_COLOR_WHITE, TRUE);
    lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

    /****************************************************/
    /* Draw static parts in menu                        */
    /****************************************************/
    lcd_putString(6,  4, "Main Menu", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
    lcd_line     (5, 12, 58, 12, LCD_COLOR_BLACK, FALSE);
    lcd_line     (5, 14, 58, 14, LCD_COLOR_BLACK, FALSE);

    lcd_putPixel (6, 21, LCD_COLOR_BLACK);
    lcd_putPixel (6, 31, LCD_COLOR_BLACK);
    lcd_putPixel (6, 41, LCD_COLOR_BLACK);
    lcd_putPixel (6, 51, LCD_COLOR_BLACK);
    lcd_putPixel (6, 61, LCD_COLOR_BLACK);
    lcd_putPixel (6, 71, LCD_COLOR_BLACK);
//    lcd_putPixel (6, 81, LCD_COLOR_BLACK);
  }

  //Erase old cursor
  lcd_line     ( 4, 16+(10*oldCursorPos), 92, 16+(10*oldCursorPos), LCD_COLOR_WHITE, FALSE);
  lcd_line     ( 4, 26+(10*oldCursorPos), 92, 26+(10*oldCursorPos), LCD_COLOR_WHITE, FALSE);
  lcd_line     ( 4, 16+(10*oldCursorPos),  4, 26+(10*oldCursorPos), LCD_COLOR_WHITE, FALSE);
  lcd_line     (92, 16+(10*oldCursorPos), 92, 26+(10*oldCursorPos), LCD_COLOR_WHITE, FALSE);

  lcd_putString(8, 18, "Clock", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(8, 28, "Slide show", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(8, 38, "Rotating globe", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(8, 48, "Bouncing ball", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(8, 58, "Clock settings", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(8, 68, "Systick rate", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
//  lcd_putString(8, 78, "TBD", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

  //Draw new cursor
  lcd_line     ( 4, 16+(10*newCursorPos), 92, 16+(10*newCursorPos), LCD_COLOR_BLACK, FALSE);
  lcd_line     ( 4, 26+(10*newCursorPos), 92, 26+(10*newCursorPos), LCD_COLOR_BLACK, FALSE);
  lcd_line     ( 4, 16+(10*newCursorPos),  4, 26+(10*newCursorPos), LCD_COLOR_BLACK, FALSE);
  lcd_line     (92, 16+(10*newCursorPos), 92, 26+(10*newCursorPos), LCD_COLOR_BLACK, FALSE);

  lcd_updateScreen(0, 95);
}

static void menuHandleKeyUp(void)
{
  uint8_t oldCursor = cursorPos;
  if (cursorPos == 0)
    cursorPos = 5;
  else
    cursorPos--;
  keyPressed = CAP_KEY_UP;
  keyPressCounter = 0;
  renderMenu(oldCursor, cursorPos);
}
static void menuHandleKeyDown(void)
{
  uint8_t oldCursor = cursorPos;
  cursorPos++;
  if (cursorPos > 5)
    cursorPos = 0;
  keyPressed = CAP_KEY_DOWN;
  keyPressCounter = 0;
  renderMenu(oldCursor, cursorPos);
}

uint8_t activateMenu(void)
{
  renderMenu(cursorPos, cursorPos);
  initSysTickCounter(5);	//turn on 5Hz tick
  inactivityCounter = 0;

  //activate cap sensors (if inactive)
  PCF8885_SLEEP_PIN_LOW;

  while(1)
  {
	//enter sleep mode
    PMU_Sleep(MCU_SLEEP, (LPC_SYSCON->PDSLEEPCFG) & ~(WDT_OSC_PD | BOD_PD));
    if (TRUE == flagCapSenseNew)
    {
      flagCapSenseNew = FALSE;

      if ((keyPressed == CAP_KEY_NONE) && (CAP_KEY_UP == (capSenseVal & (CAP_KEY_UP | CAP_KEY_DOWN))))
      {
        menuHandleKeyUp();
      }
      else if ((keyPressed == CAP_KEY_NONE) && (CAP_KEY_DOWN == (capSenseVal & (CAP_KEY_UP | CAP_KEY_DOWN))))
      {
        menuHandleKeyDown();
      }
      else if (CAP_KEY_OK == (capSenseVal & CAP_KEY_OK))
      {
        //turn of 5Hz tick
        initSysTickCounter(0);
        return cursorPos;
      }
      else if ((keyPressed != CAP_KEY_NONE) && (capSenseVal == CAP_KEY_NONE))
      {
        keyPressed = CAP_KEY_NONE;

      }
      inactivityCounter = 0;
if (0)
      {
      char message[6];
      const char toHex[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
      message[0] = toHex[capSenseVal >> 4];
      message[1] = toHex[capSenseVal & 0x0f];
      message[2] = toHex[keyPressed >> 4];
      message[3] = toHex[keyPressed & 0x0f];
      message[4] = toHex[keyPressCounter & 0x0f];
      message[5] = 0;
      lcd_putString(60, 4, message, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
      }
    }
    else if (TRUE == sysTickFlag)
    {
      sysTickFlag = FALSE;

      //check if key pressed for more than 1 sec
      if (keyPressed != CAP_KEY_NONE)
      {
        keyPressCounter++;
        if (keyPressCounter >= 4)
        {
          if ((keyPressed == CAP_KEY_UP) &&
            (CAP_KEY_UP == (capSenseVal & CAP_KEY_UP)))
          {
            menuHandleKeyUp();
            keyPressCounter = 3;
          }
          else if ((keyPressed == CAP_KEY_DOWN) &&
            (CAP_KEY_DOWN == (capSenseVal & CAP_KEY_DOWN)))
          {
            menuHandleKeyDown();
            keyPressCounter = 3;
          }
        }
      }
if (0)
      {
      char message[6];
      const char toHex[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
      message[0] = toHex[capSenseVal >> 4];
      message[1] = toHex[capSenseVal & 0x0f];
      message[2] = toHex[keyPressed >> 4];
      message[3] = toHex[keyPressed & 0x0f];
      message[4] = toHex[keyPressCounter & 0x0f];
      message[5] = 0;
      lcd_putString(60, 4, message, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
      }

      //handle inactivity counter. If no touches on 60 seconds, return 0 (= start clock application)
      inactivityCounter++;
      if (inactivityCounter > 5*60)
      {
        //turn of 5Hz tick
        initSysTickCounter(0);
        cursorPos = 0;
        return 0;
      }
    }
  }
  return 0;
}
