/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif


#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "gpio.h"
#include "s25fl004.h"
#include "ls013b4dn04.h"

#include "pic01.h"
#include "pic02.h"
#include "pic03.h"
#include "pic04.h"
#include "pic05.h"
#include "pic06.h"
#include "pic07.h"
#include "pic08.h"
#include "pic09.h"
#include "pic10.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define SYSTICK_RATE_10HZ 10
#define IMG_BUF_SZ (1220)

/******************************************************************************
 * Local variables
 *****************************************************************************/

static const char* files[] =
{
    NULL,      /* first section reserved */
    &pic01[0], /* first image */
    &pic02[0],
    &pic03[0],
    &pic04[0],
    &pic05[0],
    &pic06[0],
    &pic07[0],
    &pic08[0],
    &pic09[0],
    &pic10[0],

};

static const int fileSizes[] =
{
    0,
    sizeof(pic01),
    sizeof(pic02),
    sizeof(pic03),
    sizeof(pic04),
    sizeof(pic05),
    sizeof(pic06),
    sizeof(pic07),
    sizeof(pic08),
    sizeof(pic09),
    sizeof(pic10),

};


/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void
handle1HzLcdUpdateRate(void)
{
  static uint8_t hundredMsCount;

  hundredMsCount++;
  if (hundredMsCount >= 10)
  {
    hundredMsCount = 0;
    lcd_isr();
  }
}


/******************************************************************************
 * Main Functions
 *****************************************************************************/

int main(void) {
  uint32_t len = 0;
  int i = 0;
  uint32_t result = 0;
  uint8_t row = 0;
  uint32_t sectorSize = 0;


  GPIOInit();

  //Start sysTick counter with 100ms (10Hz) tick rate
  initSysTickCounter(SYSTICK_RATE_10HZ);

  //register systick hook
  sysTickHookRegister(handle1HzLcdUpdateRate);


  // Init the LCD
  lcd_init();

  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);


#if 1

  lcd_putString(6, 4+row*8, "Flashing...", LCD_COLOR_BLACK,
      LCD_COLOR_WHITE, 25, TRUE);row+=2;

  do {
    result = s25fl004_init();
    if (result == FALSE) {
      lcd_putString(6, 4+row*8, " Init failed", LCD_COLOR_BLACK,
          LCD_COLOR_WHITE, 25, TRUE);row++;
      break;
    }

    sectorSize = flash_getSectorSize();

    // check that the file array and file size array match in size
    if (sizeof(files) / sizeof(unsigned char*) > sizeof(fileSizes) / sizeof(int)) {
      lcd_putString(6, 4+row*8, " File array mismatch", LCD_COLOR_BLACK,
          LCD_COLOR_WHITE, 25, TRUE);row++;
      break;
    }

    // check that the flash is large enough for the images
    if ((flash_getTotalSize() / sectorSize) < sizeof(files)/sizeof(unsigned char*)) {
      lcd_putString(6, 4+row*8, " Too many images", LCD_COLOR_BLACK,
          LCD_COLOR_WHITE, 25, TRUE);row++;
      break;
    }


    for (i = 0; i < sizeof(files)/sizeof(unsigned char*); i++) {
      if (files[i] != NULL) {
        len = fileSizes[i];

        if (len > sectorSize) {
          lcd_putString(6, 4+row*8, " File too large", LCD_COLOR_BLACK,
              LCD_COLOR_WHITE, 25, TRUE);row++;
          break;
        }

        flash_erase_sector(i*sectorSize);

        result = flash_write((uint8_t*)files[i], i*sectorSize, len);

        if (result != len) {
          lcd_putString(6, 4+row*8, " Failed to write", LCD_COLOR_BLACK,
              LCD_COLOR_WHITE, 25, TRUE);row++;
          break;
        }
      }
    }


  } while (0);

  lcd_putString(6, 4+row*8, "Done", LCD_COLOR_BLACK,
      LCD_COLOR_WHITE, 25, TRUE);row++;

#else

  // a slideshow app that can be used to load the images
  // that have been written to flash

  do {
    uint32_t lastValid = 0xFFFF;
    uint32_t lastFound = FALSE;
    uint32_t nextImg = 1;
    uint8_t imgbuf[IMG_BUF_SZ];

    result = s25fl004_init();
    if (result == FALSE) {
      lcd_putString(6, 4+row*8, " Init failed", LCD_COLOR_BLACK,
          LCD_COLOR_WHITE, 25, TRUE);row++;
      break;
    }

    sectorSize = flash_getSectorSize();

    while(1) {
      result = flash_read(imgbuf, nextImg*sectorSize, IMG_BUF_SZ);
      if (result == 0) {
        lcd_putString(6, 4+row*8, " Load img failed", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);row++;
        break;
      }

      result = lcd_copyBmp(imgbuf, TRUE);
      if (!result && !lastFound) {
        lastValid = nextImg-1;
        lastFound = TRUE;
      }

      nextImg++;
      if (nextImg > lastValid) {
        nextImg = 1;
      }

      sysTickDelay(5000);
    }

  } while (0);


#endif

  return 0;
}



