/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "general.h"
#include "systick.h"
#include "ls013b4dn04.h"
#include <string.h>


/******************************************************************************
 * Defines, macros, and typedefs
 *****************************************************************************/


// Screen width
#define WIDTH 96
// Screen height
#define HEIGHT 96

#define NUM_OF_DOTS 50

// Vertex structure
typedef struct
{
  long x, y, z;
}tVertex;

/******************************************************************************
 * Public variables
 *****************************************************************************/


/******************************************************************************
 * Local variables
 *****************************************************************************/

// Transformed vertices
static tVertex *gRVtx;

static const tVertex gVtx[NUM_OF_DOTS] = {
    {-21585, 17597, -17265},
    {28493, -7790, 14183},
    {13031, 27845, 11338},
    {10822, 29162, -10304},
    {19517, -25865, -4876},
    {1283, 30949, -10687},
    {-23097, 12297, 19723},
    {-17162, 1188, 27888},
    {-29007, -1547, 15163},
    {-8182, -31729, -197},
    {11599, -28575, -11073},
    {25963, 12182, 15850},
    {106, 10547, -31023},
    {-24312, 3053, -21755},
    {9966, -18803, -24916},
    {17598, -6037, -26973},
    {23321, 17149, -15353},
    {-3265, 8867, -31376},
    {-23639, 13701, 18087},
    {-2433, -22123, -24049},
    {21284, -11349, -22179},
    {-21407, 24683, -2486},
    {-32011, -4336, 5495},
    {-20191, 22605, 12450},
    {14752, -23540, 17376},
    {-8961, -17292, -26351},
    {-18078, 13561, -23727},
    {10730, -23639, 19994},
    {-18718, 17555, 20376},
    {13626, -22837, 19144},
    {-24695, -19036, -10073},
    {11935, 22275, 20859},
    {-2182, -28801, -15474},
    {21428, -21867, -11678},
    {-19601, 21558, -14991},
    {24512, 10876, -18830},
    {12385, 27881, 11956},
    {26982, 15618, -10088},
    {-16954, 19591, 20061},
    {-6027, 22699, -22850},
    {5453, 28825, -14598},
    {-20155, -16252, -20083},
    {-15962, 11757, -26089},
    {-29175, -11289, -9750},
    {-15370, 604, -28933},
    {26009, 19868, -1575},
    {24722, -17277, -12806},
    {-4527, 25836, -19639},
    {-22224, 10442, 21697},
    {-10388, 24393, -19255}};

/******************************************************************************
 * Local functions
 *****************************************************************************/

/*
static unsigned short isqrt(unsigned long a){
	unsigned long temp;
	long e;
	unsigned long x = 0;
	if((a & 0xffff0000) != 0)
		x = 444 + a / 26743;
	else if((a & 0xff00) != 0)
		x = 21 + a / 200;
	else
		x = 1 + a / 12;
	do{
		temp = a / x;
		e = (x - temp) / 2;
		x = (x + temp) / 2;
	}
	while(e != 0);
	return (unsigned short)x;
}
 */

static short _sin(short y){
  static short s1 = 0x6487;
  static short s3 = 0x2951;
  static short s5 = 0x4f6;
  long z, prod, sum;

  z = ((long)y * y) >> 12;
  prod = (z * s5) >> 16;
  sum = s3 - prod;
  prod = (z * sum) >> 16;
  sum = s1 - prod;

  // for better accuracy, round here
  return (short)((y * sum) >> 13);
}

static short _cos(short y){
  static short c0 = 0x7fff;
  static short c2 = 0x4eea;
  static short c4 = 0x0fc4;
  long z, prod, sum;
  z = ((long)y * y) >> 12;
  prod = (z * c4) >> 16;
  sum = c2 - prod;

  // for better accuracy, round here
  prod = (z * sum) >> 15;
  return (short)(c0 - prod);
}

static short isine(short x){
  unsigned short n = (((unsigned short)x + 0x2000) >> 14) & 0x3;
  x -= n * 0x4000;
  switch(n){
  case 0:
    return _sin(x);
  case 1:
    return _cos(x);
  case 2:
    return - _sin(x);
  case 3:
    return  - _cos(x);
  }

  return -1;
}


static short icosine(short x){
  return isine(x + 0x4000);
}

static void
init()
{
  static tVertex workBuff[NUM_OF_DOTS];
  gRVtx = &workBuff[0]; //(tVertex*)malloc(sizeof(tVertex) * NUM_OF_DOTS);
}

static void rotate_z(uint32_t angle)
{
  uint32_t i;
  long x,y,ca,sa;

  ca = icosine(angle);
  sa = isine(angle);
  for (i = 0; i < NUM_OF_DOTS; i++)
  {
    x = (gRVtx[i].x * ca - gRVtx[i].y * sa) / 0x7fff;
    y = (gRVtx[i].x * sa + gRVtx[i].y * ca) / 0x7fff;
    gRVtx[i].x = x;
    gRVtx[i].y = y;
  }
}

static void rotate_y(uint32_t angle)
{
  uint32_t i;
  long x,z,ca,sa;

  ca = icosine(angle);
  sa = isine(angle);
  for (i = 0; i < NUM_OF_DOTS; i++)
  {
    z = (gRVtx[i].z * ca - gRVtx[i].x * sa) / 0x7fff;
    x = (gRVtx[i].z * sa + gRVtx[i].x * ca) / 0x7fff;
    gRVtx[i].z = z;
    gRVtx[i].x = x;
  }
}

/******************************************************************************
 * Public functions
 *****************************************************************************/

void rotatingGlobe(void)
{
  uint16_t i,j;
  uint32_t roty, rotz;

  init();
  lcd_clearScreen(LCD_COLOR_WHITE, TRUE);	//clear screen completely first time
  for(j=0; j<500; j++)
  {
    lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
    lcd_circle(96/2, 96/2, 96/3, LCD_COLOR_BLACK, FALSE);

    memcpy(gRVtx, gVtx, sizeof(tVertex) * NUM_OF_DOTS);

    rotz = (50*j) * 5;
    roty = (50*j) * 20;
    rotate_y(roty);
    rotate_z(rotz);

    for (i = 0; i < NUM_OF_DOTS; i++)
    {
      if (gRVtx[i].z >= 0)
        lcd_putPixel((int)((gRVtx[i].x * (HEIGHT / 3)) / 0x7fff + 96/2), (int)((gRVtx[i].y * (HEIGHT / 3)) / 0x7fff + 96/2), LCD_COLOR_BLACK);
    }
    lcd_updateScreen(16, 80);

    //Delay 75ms
    sysTickDelay(75);
  }
}
