/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include "type.h"
#include "general.h"
#include "application.h"
#include "adc.h"
#include "systick.h"
#include "gpio.h"
#include "ls013b4dn04.h"
#include "lpc_winfreesystem14x16.h"

#include "image_clock_96x96.h"
#include "image_ea_46x64.h"
#include "image_eco_96x96.h"
#include "image_fonts_96x96.h"
#include "image_girl_96x96.h"
#include "image_sharon_stone_96x96.h"
#include "image_sharp_logo_96x96.h"
#include "image_tiger_96x96.h"

void rotatingGlobe(void);

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define SYSTICK_RATE_10HZ 10

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void startupMessage(void)
{
  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

  /****************************************************/
  /* First screen: "Thank you for buying..."			*/
  /****************************************************/
  lcd_putString( 6,  4, "Thank you for", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString( 6, 12, "buying...", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString( 6, 24, "The ORYX board", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString(12, 32, "- powered by", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString(24, 40, "LPCXpresso", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);

  //Draw box with shade around "The ORYX Board - powered by LPCXpresso"
  lcd_line( 3, 22, 91, 22, LCD_COLOR_BLACK, FALSE);
  lcd_line(91, 22, 91, 48, LCD_COLOR_BLACK, FALSE);
  lcd_line(91, 48,  3, 48, LCD_COLOR_BLACK, FALSE);
  lcd_line( 3, 48,  3, 22, LCD_COLOR_BLACK, FALSE);
  lcd_line( 5, 49, 92, 49, LCD_COLOR_BLACK, FALSE);
  lcd_line(92, 24, 92, 49, LCD_COLOR_BLACK, TRUE);

  //Delay 3 sec
  sysTickDelay(3000);

  /****************************************************/
  /* Second screen: "Developed by Embedded Artists..."*/
  /****************************************************/
  /* Clear the LCD screen */
  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

  /* Draw EA logotype */
  lcd_copyImage16(3,3, eaImage_data, (uint8_t) eaImage_width, (uint8_t) eaImage_height, TRUE);

  /* Print message */
  lcd_putString(39, 9, "Developed", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString(63, 28, "by", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString(45, 49, "Embedded", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);
  lcd_putString(49, 57, "Artists", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 25, TRUE);

  //Delay 2 sec
  sysTickDelay(2000);

}

static uint16_t random(uint16_t maxVal)
{
  static uint32_t seed = 876552345UL;

  seed = (214013*seed + 2531011);
  return ((seed >> 16)&0x7fff) % maxVal;
}

static void randomLines(void)
{
  uint8_t x0, y0;
  uint8_t x1, y1;
  uint8_t no;
  const uint8_t CENTER_X	= 96/2;
  const uint8_t CENTER_Y	= 96/2;

  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
  for(no=0; no<80; no++)
  {
    x0 = random(96);
    y0 = random(96);

    x1 = 2*CENTER_X - x0;
    y1 = 2*CENTER_Y - y0;

    lcd_line(x0, y0, x1, y1, LCD_COLOR_BLACK, FALSE);
    lcd_updateScreen(0,95);
    //Delay 75ms
    sysTickDelay(75);
  }

  //Delay 500ms
  sysTickDelay(500);
}

static void randomCircles(void)
{
  uint8_t x0, y0;
  uint8_t radius;
  uint8_t no;

  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
  for(no=0; no<80; no++)
  {
    x0 = random(96);
    y0 = random(96);
    radius = random(20);

    lcd_circle(x0, y0, radius, LCD_COLOR_BLACK, TRUE);

    //Delay 75ms
    sysTickDelay(75);
  }

  //Delay 500ms
  sysTickDelay(500);
}

static void biggerFont(void)
{
  static uint16_t counter = 0;
  uint8_t i;

  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);
  for(i=0; i<50; i++)
  {
    lcd_copyImage16( 6, 24, &winfreesystem14x16_bits[((counter / 100) % 10) << 4], 8, 16, FALSE);
    lcd_copyImage16(14, 24, &winfreesystem14x16_bits[((counter / 10) % 10) << 4], 8, 16, FALSE);
    lcd_copyImage16(22, 24, &winfreesystem14x16_bits[(counter % 10) << 4], 8, 16, TRUE);

    counter++;

    //Delay 100ms
    sysTickDelay(100);
  }

  //Delay 1 sec
  sysTickDelay(1000);
}


static uint8_t getPicturePixel(uint8_t *pImg, uint8_t x, uint8_t y)
{
  const uint8_t bitMask[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
  return pImg[(x/8)+(y*96/8)] & bitMask[x&7];
}

static void showPictures(void)
{
  uint8_t i;

  lcd_copyImage8(0,0, &clock_96x96[4], clock_96x96[0], clock_96x96[2], FALSE, TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  lcd_copyImage8(0,0, &eco_96x96[4], eco_96x96[0], eco_96x96[2], FALSE, TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  lcd_copyImage8(0,0, &fonts_96x96[4], fonts_96x96[0], fonts_96x96[2], FALSE, TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  lcd_copyImage8(0,0, &girl_96x96[4], girl_96x96[0], girl_96x96[2], FALSE, TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  lcd_copyImage8(0,0, &sharon_stone_96x96[4], sharon_stone_96x96[0], sharon_stone_96x96[2], FALSE, TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  lcd_copyImage8(0,0, &sharp_logo_96x96[4], sharp_logo_96x96[0], sharp_logo_96x96[2], FALSE, TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  lcd_copyImage8(0,0, &tiger_96x96[4], tiger_96x96[0], tiger_96x96[2],FALSE,  TRUE);
  sysTickDelay(2000);	//Delay 2 sec

  //Transition #1
  i = 96;
  do
  {
    uint8_t colCnt;
    uint8_t rowCnt;
    uint8_t pixels;
    uint8_t pixelsA = 0;
    uint8_t pixelsB = 0;
    uint8_t *pImgA, *pImgB;

    pImgA = (uint8_t*)&tiger_96x96[4];
    pImgB = (uint8_t*)&clock_96x96[4];
    for (rowCnt = 0; rowCnt < 96; rowCnt++)
    {
      for (colCnt = 0; colCnt < 96; colCnt++)
      {
        if ((colCnt & 0x07) == 0)
        {
          pixelsA = *pImgA++;
          pixelsB = *pImgB++;
        }
        if (colCnt < i)
          pixels = pixelsA;
        else
          pixels = pixelsB;
        if (pixels & 0x80)
          lcd_putPixel(colCnt, rowCnt, LCD_COLOR_BLACK);
        else
          lcd_putPixel(colCnt, rowCnt, LCD_COLOR_WHITE);
        pixelsA <<= 1;
        pixelsB <<= 1;
      }
    }
    lcd_updateScreen(0, 95);

  } while (i-- > 0);

  //Transition #2
  sysTickDelay(1000);	//Delay 1 sec
  for(i=0; i<=96; i++)
  {
    lcd_copyImage8(0, 0, &tiger_96x96[4+(96*96/8)-(i*96/8)], 96, i, FALSE, FALSE);
    lcd_copyImage8(0, i, &clock_96x96[4+(i*96/8)], 96, 96-i, FALSE, TRUE);
  }

  //Transition #3
  sysTickDelay(1000);	//Delay 1 sec
  i = 96;
  do
  {
    uint8_t colCnt;
    uint8_t rowCnt;
    uint8_t pixel;

    for (colCnt = 0; colCnt < i; colCnt++)
    {
      for (rowCnt = 0; rowCnt < 96; rowCnt++)
      {
        pixel = getPicturePixel((uint8_t*)&tiger_96x96[4],96+colCnt-i,rowCnt);

        if (pixel)
          lcd_putPixel(colCnt, rowCnt, LCD_COLOR_BLACK);
        else
          lcd_putPixel(colCnt, rowCnt, LCD_COLOR_WHITE);
      }
    }
    for (colCnt = i; colCnt < 96; colCnt++)
    {
      for (rowCnt = 0; rowCnt < 96; rowCnt++)
      {
        pixel = getPicturePixel((uint8_t*)&clock_96x96[4],colCnt-i,rowCnt);

        if (pixel)
          lcd_putPixel(colCnt, rowCnt, LCD_COLOR_BLACK);
        else
          lcd_putPixel(colCnt, rowCnt, LCD_COLOR_WHITE);
      }
    }
    lcd_updateScreen(0, 95);
  } while (i-- > 0);

  //Transition #4
  sysTickDelay(1000);	//Delay 1 sec
  for(i=0; i<=96; i++)
  {
    lcd_copyImage8(0, 0, &tiger_96x96[4+(96*96/8)-(i*96/8)], 96, i, FALSE, FALSE);
    lcd_copyImage8(0, i, &clock_96x96[4], 96, 96-i, FALSE, TRUE);
  }

  sysTickDelay(1000);	//Delay 1 sec
}

static void handle1HzLcdUpdateRate(void)
{
  static uint8_t hundredMsCount;

  hundredMsCount++;
  if (hundredMsCount >= 10)
  {
    hundredMsCount = 0;
    EA_LED_ON;
    lcd_isr();
    EA_LED_OFF;
  }
}

/******************************************************************************
 * Main functions
 *****************************************************************************/

int main(void) {
  GPIOInit();

  /* Set LED pin (PIO0_7) to output */
  EA_LED_INIT;  //LED pin
  EA_LED_OFF;

  PWR_ADC_INIT; //ADC power control pin
  PWR_ADC_OFF;

  PWR_I2C_INIT; //I2C pull-up control pin
  PWR_I2C_ON;

  //Init the LCD
  lcd_init();

  //Start sysTick counter with 100ms (10Hz) tick rate
  initSysTickCounter(SYSTICK_RATE_10HZ);

  //register systick hook
  sysTickHookRegister(handle1HzLcdUpdateRate);

  //Start graphics demo
  while(1)
  {
    startupMessage();

    randomLines();

    randomCircles();

    biggerFont();

    showPictures();

    rotatingGlobe();
  }

  return 0 ;
}

/******************************************************************************
 **                            End Of File
 ******************************************************************************/

