/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif



#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "gpio.h"
#include "adc.h"
#include "adxl345.h"
#include "s25fl004.h"
#include "ls013b4dn04.h"
#include "i2c.h"
#include "lm75b.h"
#include "pcf8885.h"
#include "pmu.h"
#include "pwrapi.h"
#include "clkconfig.h"
#include "wdt.h"
#include "systick.h"

/******************************************************************************
 * Main function
 *****************************************************************************/

int main(void)
{
  GPIOInit();

  //Set LED pin (PIO0_7) to output and turn LED off
  EA_LED_INIT;
  EA_LED_OFF;

  //Turn on-chip USB peripheral block off
  LPC_SYSCON->PDRUNCFG |= (1 << 10);	//Power-down USB PHY
  LPC_SYSCON->PDRUNCFG |= (1 <<  8);	//Power-down USB PLL
  //  LPC_SYSCON->PDRUNCFG |= (IRC_OUT_PD | IRC_PD);  //Power-down IRC

  PMU_Init();

  //Turn LCD off
  PWR_LCD_INIT;
  PWR_LCD_OFF;

  //Turn SPI-FLASH off
  PWR_FLASH_INIT;
  PWR_FLASH_OFF;

  //Init and enable I2C pull-up resistors
  PWR_I2C_INIT;
  PWR_I2C_ON;

  if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
  {
    while ( 1 );        /* Fatal error */
  }

  //put capacitive sensor to sleep
  pcf8885_init();
  PCF8885_SLEEP_PIN_HIGH;

  //Init accelerometer
  adxl345_init();
  adxl345_setPowerControl(0x00);

  //Init pin that controls voltage to trimming potentiometer
  PWR_ADC_INIT;
  PWR_ADC_OFF;

  /* Initialize ADC  */
  ADCInit( ADC_CLK );

  //Init the temperature sensor
  lm75b_config(LM75B_CFG_NORMAL);

  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2011 Embedded Artists AB");
  printf("\n*");
  printf("\n* Demo application for printf() via SWD (semihosting),");
  printf("\n* Systick, temperature sensor and ADC.");
  printf("\n*");
  printf("\n*********************************************************\n");

  //Init systick counter (3HZ is lowest possible rate)
  initSysTickCounter(3);

  while(1)
  {
    uint16_t tempVal, adcVal;

    //enter sleep mode (will wake up after every systick interrupt)
    //Cannot enter deep sleep or power down when using semihosting
    PMU_Sleep(MCU_SLEEP, 0);

    //Read temperature & ADC and print out result
    tempVal = lm75b_readTemp();

    //read ADC input
    PWR_ADC_ON;
    adcVal = ADCRead(7);
//Keep trimming pot power on always
//    PWR_ADC_OFF;

    printf("ADC= %d, temp= %d.%d\n", adcVal, (tempVal+50) / 100, ((tempVal%100) + 5) / 10);
  }

  return 0 ;
}



