/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/


/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "gpio.h"
#include "adc.h"
#include "ls013b4dn04.h"
#include "s25fl004.h"
#include "i2c.h"
#include "pcf8523.h"
#include "pcf8885.h"
#include "adxl345.h"
#include "lm75b.h"
#include "pwrapi.h"
#include "pmu.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define USE_SYSTICK
#define USE_DISPLAY_TIMEOUT

/******************************************************************************
 * Local functions
 *****************************************************************************/

/*
static void restoreIoAfterSleep(void)
{
  SSP_IOConfig(0);
  SSP_Init(0);
}
*/

/******************************************************************************
 * Main functions
 *****************************************************************************/

int main(void)
{
  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD | ADC_PD);
  LPC_SYSCON->PDRUNCFG |= (SYS_OSC_PD);

  GPIOInit();

  prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);

  //Set LED pin (PIO0_7) to output and turn LED off
  EA_LED_INIT;
  EA_LED_OFF;

  //Turn on-chip USB peripheral block off
  LPC_SYSCON->PDRUNCFG |= (1 << 10);	//Power-down USB PHY
  LPC_SYSCON->PDRUNCFG |= (1 <<  8);	//Power-down USB PLL
  //  LPC_SYSCON->PDRUNCFG |= (IRC_OUT_PD | IRC_PD);  //Power-down IRC

  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD);

  LPC_SYSCON->STARTERP0 = (0x1<<2);			//Flex IRQ2 (CapSense irq) can startup mcu
  LPC_SYSCON->STARTERP1 = (1<<12) | (1<<13);	//WDT and BOD can startup mcu

  //Turn LCD on
  PWR_LCD_INIT;
  PWR_LCD_ON;

  //Turn SPI-FLASH off
  PWR_FLASH_INIT;
  PWR_FLASH_OFF;

  //Init and enable I2C pull-up resistors
  PWR_I2C_INIT;
  PWR_I2C_ON;

  if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
  {
    while ( 1 );        /* Fatal error */
  }

  //Init capacitive sensors
  pcf8885_init();
  PCF8885_SLEEP_PIN_LOW;

  //Init accelerometer
  adxl345_init();
  adxl345_setPowerControl(0x00);

  //Init pin that controls voltage to trimming potentiometer
  PWR_ADC_INIT;
  PWR_ADC_OFF;

  //Init the temperature sensor in power down mode
  lm75b_config(LM75B_CFG_SHUTDOWN);


  lcd_init();
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);
  lcd_putString(6,  4, "Cap sense demo", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_line     (5, 12, 90, 12, LCD_COLOR_BLACK, FALSE);
  lcd_line     (5, 14, 90, 14, LCD_COLOR_BLACK, FALSE);
  lcd_putString(3, 80, "5Hz lcd update", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

  lcd_putString(10, 20, "0", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(10, 30, "1", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(10, 40, "2", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(10, 50, "3", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(10, 60, "4", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(20, 60, "5", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(30, 60, "6", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(40, 60, "7", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

  initSysTickCounter(5);	//turn on 5Hz tick

  //Enter forever loop...
  while(1)
  {
    //		prepareIoForSleep();
    //		PCF8885_SLEEP_PIN_LOW;
    //		LPC_IOCON->TMS_PIO0_12	= (1<<0) | (2<<3) | (1<<7);  //GPIO (not TMS), pullup on cap-sense irq pin (PIO0_12)

    PMU_Sleep(MCU_SLEEP, (BOD_PD | WDT_OSC_PD)); //Power down BOD and WDT oscillator
    //		restoreIoAfterSleep();

    //Check if cap sense interrupt (is the only one possible, but check anyways)
    if (TRUE == flagCapSenseNew)
    {
      flagCapSenseNew = FALSE;

      if (capSenseVal & CAP_KEY_0)
        lcd_putString(10, 20, "0", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(10, 20, "0", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_1)
        lcd_putString(10, 30, "1", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(10, 30, "1", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_2)
        lcd_putString(10, 40, "2", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(10, 40, "2", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_3)
        lcd_putString(10, 50, "3", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(10, 50, "3", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_4)
        lcd_putString(10, 60, "4", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(10, 60, "4", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_5)
        lcd_putString(20, 60, "5", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(20, 60, "5", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_6)
        lcd_putString(30, 60, "6", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(30, 60, "6", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      if (capSenseVal & CAP_KEY_7)
        lcd_putString(40, 60, "7", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, FALSE);
      else
        lcd_putString(40, 60, "7", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      lcd_update_vcom();
      lcd_updateScreen(0, 95);
    }

    //systick/wdt wakeup
    if (TRUE == sysTickFlag)
    {
      sysTickFlag = FALSE;
      //Update VCOM on LCD
      lcd_isr();
    }
  }

  return 0 ;
}

