#include "LPC11Uxx.h"
#include "type.h"
#include "general.h"

volatile uint32_t sysTickCounter;
volatile uint8_t  sysTickFlag;
static void (*sysTickPFunc)(void) = NULL;
static uint16_t  sysTickRate;

/*****************************************************************************
** Function name:		initSysTickCounter
**
** Descriptions:		initialize the SysTick counted
**
** parameters:			SysTick rate in hertz
** Returned value:		None
**
*****************************************************************************/
void initSysTickCounter(uint16_t hzTick) {
	sysTickPFunc=NULL;

	sysTickRate = hzTick;
	if (hzTick == 0)
		/* Disable SysTick Counter */
		SysTick->CTRL &= ~(0x1<<0);
	else
	{
		SysTick_Config(SystemCoreClock / hzTick); //function call also enable SysTick counter
	}
}
/*****************************************************************************
** Function name:		SysTick_Handler
**
** Descriptions:		ISR for SysTick irq
**
** parameters:			None
** Returned value:	None
**
*****************************************************************************/
void SysTick_Handler(void) {
	sysTickFlag = TRUE;
	sysTickCounter++;

	//call hook if registered
	if (NULL != sysTickPFunc)
		sysTickPFunc();
}

/*****************************************************************************
** Function name:		sysTickHookRegister
**
** Descriptions:
**
** parameters:
** Returned value:	None
**
*****************************************************************************/
void sysTickHookRegister(void (*pFunc)(void))
{
	sysTickPFunc = pFunc;
}

/*****************************************************************************
** Function name:		sysTickDelay
**
** Descriptions:
**
** parameters:
** Returned value:	None
**
*****************************************************************************/
void sysTickDelay(uint32_t msDelay)
{
	uint32_t value = (msDelay * sysTickRate) / 1000;

	if (value)
	{
		value += sysTickCounter;
		while (sysTickCounter < value)
			;

	}
}

/******************************************************************************
**                            End Of File
******************************************************************************/

