/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

#ifndef __LS013B4DN04_H
#define __LS013B4DN04_H

#include "ssp.h"

extern uint8_t lcdSpiActive;

#define PWR_LCD_INIT GPIOSetDir( 1, 14, 1 )
#define PWR_LCD_ON   GPIOSetBitValue( 1, 14, 0 )
#define PWR_LCD_OFF  GPIOSetBitValue( 1, 14, 1 )


void lcd_init (void);
void lcd_updateScreen(uint8_t startLine, uint8_t endLine);
void lcd_isr (void);
void lcd_update_vcom (void);

#define LCD_DISPLAY_WIDTH  96
#define LCD_DISPLAY_HEIGHT 96

typedef enum
{
	LCD_COLOR_WHITE,
	LCD_COLOR_BLACK
} lcd_color_t;


void lcd_putPixel(uint8_t x, uint8_t y, lcd_color_t color);
void lcd_line(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, lcd_color_t color, uint8_t updateFlag);
void lcd_circle(uint8_t x0, uint8_t y0, uint8_t r, lcd_color_t color, uint8_t updateFlag);
void lcd_rect(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, lcd_color_t color);
void lcd_fillRect(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, lcd_color_t color);
void lcd_clearScreen(lcd_color_t color, uint8_t updateFlag);
void lcd_putString(uint8_t x, uint8_t y, char *pStr, lcd_color_t fb, lcd_color_t bg, uint8_t msDelay, uint8_t updateFlag);
uint8_t lcd_putChar(uint8_t x, uint8_t y, uint8_t ch, lcd_color_t fb, lcd_color_t bg);
void lcd_copyImage16 (uint8_t x, uint8_t y, const uint16_t * img, uint8_t width, uint8_t height, uint8_t updateFlag);
void lcd_copyImage8(uint8_t x, uint8_t y, const uint8_t * img, uint8_t width, uint8_t height,
     uint8_t flipY, uint8_t updateFlag);
void lcd_copyImageFull(const uint8_t * img, uint8_t flipY, uint8_t updateFlag);
uint32_t lcd_copyBmp(uint8_t * bmp, uint8_t updateFlag);
uint32_t lcd_getBmpOffset(uint8_t* bmp, uint16_t* pOff);


#endif /* end __LS013B4DN04_H */
/****************************************************************************
**                            End Of File
*****************************************************************************/
