/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "type.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define LM75B_I2C_ADDR	(0x48)
#define LM75B_REG_TEMP	0x00
#define LM75B_REG_CMD	0x01


/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/


/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize LM75B Driver
 *
 *****************************************************************************/
void lm75b_init (void)
{

}

/******************************************************************************
 *
 * Description:
 *    Read the temperature
 *
 * Params: None
 *
 * Returns:
 *   The measured temperature x 100, i.e. 26.50 degrees returned as 2650
 *
 *****************************************************************************/
int32_t lm75b_readTemp(void)
{
    uint8_t cmd = LM75B_REG_TEMP;
    uint8_t temp[2];
    int16_t t = 0;


    //                 I2C_write_read(i2c-addr,      *pBuf, len, *pBuf2, len2
    if (I2C_CODE_OK == I2C_write_read(LM75B_I2C_ADDR, &cmd,   1,   temp,   2))
    {
        /* 11 MSB bits used. Celcius is calculated as Temp data * 1/8 */
        t = ((temp[0] << 8) | (temp[1]));

        return ((t * 100) >> 8);
    }
    else
    	return 0xffff;
}

/******************************************************************************
 *
 * Description:
 *    Write to config register of LM75B
 *
 * Params: Config byte
 *
 * Returns: None
 *
 *****************************************************************************/
int8_t lm75b_config(int8_t config)
{
    uint8_t cmd[2];


    cmd[0] = LM75B_REG_CMD;
    cmd[1] = config;
    //     I2C_write_read(i2c-addr,      *pBuf, len, *pBuf2, len2
    return I2C_write_read(LM75B_I2C_ADDR, &cmd[0],   2,   NULL,   0);
}


