/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "LPC11Uxx.h"
#include "type.h"
#include "general.h"
#include "application.h"
#include "gpio.h"
#include "i2c.h"
#include "pcf8885.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define PCF8885_I2C_ADDR	(0x20)


/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/
volatile uint8_t flagCapSenseNew;
uint8_t capSenseVal;

/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Read SENS register
 *
 *****************************************************************************/
uint8_t pcf8885_readSense (void)
{
  static uint8_t retVal;

  //I2C_write_read(i2c-addr,      *pBuf, len,  *pBuf2, len2
  I2C_write_read(PCF8885_I2C_ADDR, NULL,   0, &retVal,   1);

  return retVal;
}

/******************************************************************************
 *
 * Description:
 *    Write command
 *
 *****************************************************************************/
int8_t pcf8885_command (uint8_t command)
{
  uint8_t cmd;

  cmd = command;

  //I2C_write_read(i2c-addr,             *pBuf, len, *pBuf2, len2
  return I2C_write_read(PCF8885_I2C_ADDR, &cmd,   1,   NULL,   0);
}

/******************************************************************************
 *
 * Description:
 *    Write register
 *
 *****************************************************************************/
int8_t pcf8885_register_write (uint8_t regAddr, uint8_t regVal)
{
  uint8_t cmd[2];

  cmd[0] = regAddr;
  cmd[1] = regVal;

  //     I2C_write_read(i2c-addr,            *pBuf, len, *pBuf2, len2
  return I2C_write_read(PCF8885_I2C_ADDR, cmd,   2,   NULL,   0);
}
/******************************************************************************
 *
 * Description:
 *    Read a register
 *
 *****************************************************************************/
uint8_t pcf8885_register_read(uint8_t regAddr)
{
  uint8_t cmd = regAddr;
  uint8_t retVal;

  //I2C_write_read(i2c-addr,      *pBuf, len,   *pBuf2, len2
  I2C_write_read(PCF8885_I2C_ADDR, &cmd,   1,  &retVal,  1);
  return retVal;
}

/*****************************************************************************
 ** Function name:		FLEX_INT2_IRQHandler
 **
 ** Descriptions:		Use GPIO0_12 pin as interrupt source
 **
 ** parameters:			None
 **
 ** Returned value:		None
 **
 *****************************************************************************/
void FLEX_INT2_IRQHandler(void)
{
  PWR_SIGNAL_IRQ_START;
  if ( LPC_GPIO_PIN_INT->IST & (0x1<<2) )
  {
    if ( LPC_GPIO_PIN_INT->FALL & (0x1<<2) )
    {
      //check irq from PCF8885
      PWR_I2C_ON;
      capSenseVal = pcf8885_readSense();
      PWR_I2C_OFF;
      flagCapSenseNew = TRUE;

      LPC_GPIO_PIN_INT->FALL = 0x1<<2;
    }
    LPC_GPIO_PIN_INT->IST = 0x1<<2;
  }
  PWR_SIGNAL_IRQ_END;
  return;
}

/******************************************************************************
 *
 * Description:
 *    Initialize the PCF8885 Driver
 *
 *****************************************************************************/
void pcf8885_init (void)
{
  //Set PCF8885 pin 'INTN' as input (PIO0_12)
  GPIOSetDir( 0, 12, 0 );

  //Set PCF8885 pin 'SLEEP' as output (PIO1_15)
  GPIOSetDir( 1, 15, 1 );
  PCF8885_SLEEP_PIN_LOW;

  pcf8885_command(PCF8885_COM_RESET);
  pcf8885_command(PCF8885_COM_CLR_INT);
  pcf8885_command(PCF8885_COM_WAKEUP);

  //Set clock to 2.2kHz sampling
  pcf8885_register_write(PCF8885_REG_WR_CLOCK, 0x14); //set to 0x12 for 1.6kHz sampling frequency

  //initialize P0.12 as edge sensitive interrupt, active falling edge
  GPIOSetFlexInterrupt( CHANNEL2, PORT0, 12, 0, 0 );
  NVIC_SetPriority(FLEX_INT2_IRQn, 2);
  LPC_SYSCON->STARTERP0 |= (0x1<<2);  //This interrupt can wakeup the mcu from power down mode
  GPIOFlexIntEnable( CHANNEL2, 0 );
}

