/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "LPC11Uxx.h"
#include "type.h"
#include "general.h"
#include "pwrapi.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define KEEP_SWD_ACTIVE

/******************************************************************************
 * Public functions
 *****************************************************************************/

/*****************************************************************************
** Function name:		prepareIoForSleep
**
** Descriptions:		Set
**
** parameters:			flasg to keep spi0 and clockout pins active (will increase current consumption)
** Returned value:		None
**
*****************************************************************************/
void prepareIoForSleep(uint8_t keepSpi0Active, uint8_t keepClockOutActive)
{
  	//Set pins
  	LPC_IOCON->RESET_PIO0_0		 = (0<<0) | (0<<3);  //RESET input, no pullup/down
  	if (keepClockOutActive)
  		LPC_IOCON->PIO0_1        = (1<<0) | (0<<3);  //CLKOUT, no pullup/down
  	else
  		LPC_IOCON->PIO0_1        = (0<<0) | (0<<3);  //GPIO, no pullup/down
	LPC_IOCON->PIO0_2        	 = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_3            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_4            = (1<<0) | (0<<3);  //I2C-SCL, normal mode
  	LPC_IOCON->PIO0_5            = (1<<0) | (0<<3);  //I2C-SDA, normal mode
  	LPC_IOCON->PIO0_6            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_7            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_8            = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	if (keepSpi0Active)
  		LPC_IOCON->PIO0_9        = (1<<0) | (0<<3);  //SPI0-MOSI, no pullup/down
  	else
  		LPC_IOCON->PIO0_9        = (0<<0) | (0<<3);  //GPIO, no pullup/down
#ifdef KEEP_SWD_ACTIVE
  	LPC_IOCON->SWCLK_PIO0_10	 = (0<<0) | (2<<3);  //SWCLK, pullup active
#else
  	LPC_IOCON->SWCLK_PIO0_10	 = (1<<0) | (0<<3);  //GPIO (not SWCLK), no pullup/down
#endif
    LPC_IOCON->TDI_PIO0_11	     = (1<<0) | (2<<3) | (1<<7);  //GPIO (not TDI), pullup on rtc irq input pin
 	LPC_IOCON->TMS_PIO0_12	     = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TMS), no pullup/down
  	LPC_IOCON->TDO_PIO0_13       = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TDO), no pullup/down
  	LPC_IOCON->TRST_PIO0_14      = (1<<0) | (0<<3) | (1<<7);  //GPIO (not TRST), no pullup/down
#ifdef KEEP_SWD_ACTIVE
 	LPC_IOCON->SWDIO_PIO0_15	 = (0<<0) | (2<<3) | (1<<7);  //SWDIO), pullup active
#else
 	LPC_IOCON->SWDIO_PIO0_15	 = (1<<0) | (0<<3) | (1<<7);  //GPIO (not SWDIO), no pullup/down
#endif
  	LPC_IOCON->PIO0_16           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_17           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_18           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_19           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_20           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_21           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_22           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO0_23           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_0            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_1            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_2            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_3            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_4            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_5            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_6            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_7            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_8            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_9            = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_10           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_11           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_12           = (0<<0) | (0<<3) | (1<<7);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_13           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_14           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_15           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_16           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_17           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_18           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_19           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_20           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_21           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_22           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_23           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_24           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_25           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_26           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_27           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_28           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	if (keepSpi0Active)
  		LPC_IOCON->PIO1_29       = (1<<0) | (0<<3);  //SPI0-SCK, no pullup/down
  	else
  		LPC_IOCON->PIO1_29       = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_30           = (0<<0) | (0<<3);  //GPIO, no pullup/down
  	LPC_IOCON->PIO1_31           = (0<<0) | (0<<3);  //GPIO, no pullup/down

  	//P0.11/12/13/14 (irq:s) and P0.16 (wakeup) are inputs
  	LPC_GPIO->DIR[0] = ~((1<<11)|(1<<12)|(1<<13)|(1<<14)|(1<<16));
  	LPC_GPIO->DIR[1] = 0xFFFFFFFF;

  	//All pins output low, except P0.6/(P0.8 LCD ENA)/P1.13/(P1.14 LCD PWR)/P1.15/P1.16/P1.23 that shall be high
  	LPC_GPIO->CLR[0] = ~((1<<6)|(1<<8));
  	LPC_GPIO->CLR[1] = ~((1<<13)|(1<<15)|(1<<16));
  	LPC_GPIO->SET[0] = ((1<<6)|(1<<8));
  	LPC_GPIO->SET[1] = (1<<13)|(1<<15)|(1<<16);
//  LPC_SYSCON->PDSLEEPCFG = (1<<6); //BOD powered, WDT powered down
//    LPC_SYSCON->PDSLEEPCFG = (1<<3)|(1<<6); //BOD and WDT powered down
}

/******************************************************************************
**                            End Of File
******************************************************************************/

